package com.qmth.eds.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.eds.common.contant.SystemConstant;
import com.qmth.eds.common.entity.CloudMarkingExam;
import com.qmth.eds.common.enums.ExceptionResultEnum;
import com.qmth.eds.common.tools.CloudMarkingUtil;
import com.qmth.eds.common.util.ServletUtil;
import com.qmth.eds.mapper.CloudMarkingExamMapper;
import com.qmth.eds.service.CloudMarkingExamService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class CloudMarkingExamServiceImpl extends ServiceImpl<CloudMarkingExamMapper, CloudMarkingExam> implements CloudMarkingExamService {

    @Resource
    CloudMarkingUtil cloudMarkingUtil;

    @Override
    public List<CloudMarkingExam> listExam() {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        QueryWrapper<CloudMarkingExam> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CloudMarkingExam::getSchoolId, schoolId)
                .orderByDesc(CloudMarkingExam::getExamId);
        return this.list(queryWrapper);
    }

    @Transactional
    @Override
    public List<CloudMarkingExam> syncExam() {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        UpdateWrapper<CloudMarkingExam> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(CloudMarkingExam::getSchoolId, schoolId);
        this.remove(updateWrapper);

        List<JSONObject> objectList = cloudMarkingUtil.getExam(SystemConstant.PAGE_NUMBER_MIN, SystemConstant.PAGE_SIZE);
        if (objectList.isEmpty()) {
            throw ExceptionResultEnum.ERROR.exception("未查询到考试");
        }
        List<CloudMarkingExam> markingExams = objectList.stream().map(m -> {
            CloudMarkingExam cloudMarkingExam = new CloudMarkingExam();
            cloudMarkingExam.setId(SystemConstant.getDbUuid());
            cloudMarkingExam.setSchoolId(schoolId);
            cloudMarkingExam.setExamId(m.getInteger("id"));
            cloudMarkingExam.setExamName(m.getString("name"));
            return cloudMarkingExam;
        }).collect(Collectors.toList());
        if (!markingExams.isEmpty()) {
            this.saveBatch(markingExams);
        }
        return markingExams;
    }
}
